@@ -1,25 +1,22 @@
-from collections import namedtuple
 import unittest
+from collections import namedtuple
 
 from parse_this import create_parser, parse_class
-from test.utils import captured_output
-from parse_this.core import (
-    _get_args_and_defaults,
-    NoDefault,
-    _get_default_help_message,
-    Self,
-    _get_parseable_methods,
-    Class,
-    _prepare_doc,
-    _get_arg_parser,
-    _get_args_to_parse,
-    ParseThisError,
-    _check_types,
-    _get_parser_call_method,
+from parse_this.args import _get_args_and_defaults, _get_args_to_parse
+from parse_this.call import (
     _call,
     _call_method_from_namespace,
-    identity_type,
+    _get_parser_call_method,
 )
+from parse_this.exception import ParseThisException
+from parse_this.help.description import _get_default_help_message, prepare_doc
+from parse_this.parsing import (
+    _get_arg_parser,
+    _get_parseable_methods,
+)
+from parse_this.types import _check_types
+from parse_this.values import _NO_DEFAULT
+from test.utils import captured_output
 
 
 def no_docstring():
@@ -32,51 +29,51 @@ def with_args(a, b):
 
 @parse_class()
 class Parseable(object):
-    @create_parser(Self, int)
-    def __init__(self, a):
+    @create_parser()
+    def __init__(self, a: int):
         self._a = a
 
-    @create_parser(Self, int)
-    def _private_method(self, b):
+    @create_parser()
+    def _private_method(self, b: int):
         return self._a * b
 
-    def not_parseable(self, c):
+    def not_parseable(self, c: int):
         return self._a * c
 
-    @create_parser(Self, int)
-    def parseable(self, d):
+    @create_parser()
+    def parseable(self, d: int):
         return self._a * d
 
     @classmethod
-    @create_parser(Class, int)
-    def cls_method(cls, e):
+    @create_parser()
+    def cls_method(cls, e: int):
         return e * e
 
 
 @parse_class(parse_private=True)
 class ParseableWithPrivateMethod(object):
-    @create_parser(Self, int)
-    def __init__(self, a):
+    @create_parser()
+    def __init__(self, a: int):
         self._a = a
 
-    @create_parser(Self, int)
-    def _private_method(self, b):
+    @create_parser()
+    def _private_method(self, b: int):
         return self._a * b
 
-    def not_parseable(self, c):
+    def not_parseable(self, c: int):
         return self._a * c
 
-    @create_parser(Self, int)
-    def parseable(self, d):
+    @create_parser()
+    def parseable(self, d: int):
         return self._a * d
 
     @classmethod
-    @create_parser(Class, int)
-    def cls_method(cls, e):
+    @create_parser()
+    def cls_method(cls, e: int):
         return e * e
 
 
-def blank_line_in_wrong_place(one, two):
+def blank_line_in_wrong_place(one: int, two: int):
     """I put the blank line after arguments ...
 
     Args:
@@ -87,7 +84,7 @@ def blank_line_in_wrong_place(one, two):
     return one * two
 
 
-def parse_me_full_docstring(one, two, three=12):
+def parse_me_full_docstring(one: str, two: int, three: int = 12):
     """Could use some parsing.
 
     Args:
@@ -102,11 +99,11 @@ def parse_me_full_docstring(one, two, three=12):
     return one * two, three * three
 
 
-def parse_me_no_docstring(one, two, three):
+def parse_me_no_docstring(one: int, two: int, three: int):
     return one * two, three * three
 
 
-def multiline_docstring(one, two, three):
+def multiline_docstring(one: int, two: int, three: int):
     """I am a sneaky function.
 
     Args:
@@ -121,7 +118,7 @@ def multiline_docstring(one, two, three):
     return one * two, three * three
 
 
-def different_delimiter_charsiter(one, two, three):
+def different_delimiter_chars(one: int, two: int, three: int):
     """I am a sneaky function.
 
     Args:
@@ -136,37 +133,38 @@ def different_delimiter_charsiter(one, two, three):
     return one * two, three * three
 
 
-@create_parser(str, int)
-def concatenate_string(string, nb_concat):
+@create_parser()
+def concatenate_string(string: str, nb_concat: int):
     return string * nb_concat
 
 
-@create_parser(int, str)
-def has_none_default_value(a, b=None):
+@create_parser()
+def has_none_default_value(a: int, b: str = None):
     return a, b
 
 
-@create_parser(int)
-def has_flags(a, b=False):
+@create_parser()
+def has_flags(a: int, b: bool = False):
     return a, b
 
 
-@create_parser(bool)
-def has_bool_arguments(a):
+@create_parser()
+def has_bool_arguments(a: bool):
     return a
 
 
 class TestCore(unittest.TestCase):
-    def test_identity_type(self):
-        self.assertEqual(None, identity_type(None))
-
     def test_get_args_and_defaults_fill_no_default(self):
         args_and_defaults = _get_args_and_defaults(
             ["first", "second", "third"], ("default_value",)
         )
         self.assertListEqual(
             args_and_defaults,
-            [("first", NoDefault), ("second", NoDefault), ("third", "default_value")],
+            [
+                ("first", _NO_DEFAULT),
+                ("second", _NO_DEFAULT),
+                ("third", "default_value"),
+            ],
         )
 
     def test_get_args_and_defaults_no_args(self):
@@ -175,7 +173,7 @@ class TestCore(unittest.TestCase):
     def test_get_args_and_defaults_no_default(self):
         self.assertListEqual(
             _get_args_and_defaults(["first", "second"], ()),
-            [("first", NoDefault), ("second", NoDefault)],
+            [("first", _NO_DEFAULT), ("second", _NO_DEFAULT)],
         )
 
     def test_get_default_help_message_no_docstring(self):
@@ -203,7 +201,7 @@ class TestCore(unittest.TestCase):
         self.assertNotIn("cls_method", method_to_parser.keys())
 
     def test_prepare_doc_blank_line_in_wrong_place(self):
-        (description, help_msg) = _prepare_doc(
+        (description, help_msg) = prepare_doc(
             blank_line_in_wrong_place, ["one", "two"], ":"
         )
         self.assertEqual(description, "I put the blank line after arguments ...")
@@ -212,7 +210,7 @@ class TestCore(unittest.TestCase):
         )
 
     def test_prepare_doc_full_docstring(self):
-        (description, help_msg) = _prepare_doc(
+        (description, help_msg) = prepare_doc(
             parse_me_full_docstring, ["one", "two", "three"], ":"
         )
         self.assertEqual(description, "Could use some parsing.")
@@ -226,7 +224,7 @@ class TestCore(unittest.TestCase):
         )
 
     def test_prepare_doc_no_docstring(self):
-        (description, help_msg) = _prepare_doc(
+        (description, help_msg) = prepare_doc(
             parse_me_no_docstring, ["one", "two", "three"], ":"
         )
         self.assertEqual(description, "Argument parsing for parse_me_no_docstring")
@@ -240,7 +238,7 @@ class TestCore(unittest.TestCase):
         )
 
     def test_prepare_doc_will_you_dare(self):
-        (description, help_msg) = _prepare_doc(
+        (description, help_msg) = prepare_doc(
             multiline_docstring, ["one", "two", "three"], ":"
         )
         self.assertEqual(description, "I am a sneaky function.")
@@ -255,8 +253,8 @@ class TestCore(unittest.TestCase):
         )
 
     def test_prepare_doc_delimiter_chars(self):
-        (description, help_msg) = _prepare_doc(
-            different_delimiter_charsiter, ["one", "two", "three"], "--"
+        (description, help_msg) = prepare_doc(
+            different_delimiter_chars, ["one", "two", "three"], "--"
         )
         self.assertEqual(description, "I am a sneaky function.")
         self.assertEqual(
@@ -278,10 +276,10 @@ class TestCore(unittest.TestCase):
         self.assertEqual(has_flags.parser.call(args=["12", "--b"]), (12, True))
 
     def test_get_arg_parser_none_default_value_without_type(self):
-        with self.assertRaises(ParseThisError):
+        with self.assertRaises(ParseThisException):
 
             @create_parser(int)
-            def have_none_default_value(a, b=None):
+            def have_none_default_value(a: int, b=None):
                 pass
 
     def test_get_arg_parser_with_none_default_value(self):
@@ -290,11 +288,23 @@ class TestCore(unittest.TestCase):
             has_none_default_value.parser.call(args=["12", "--b", "yes"]), (12, "yes")
         )
 
+    def test_get_arg_parser_annotation_take_precedence(self):
+        parser = _get_arg_parser(
+            parse_me_full_docstring,
+            {"one": int, "two": int, "three": int},
+            [("one", _NO_DEFAULT), ("two", _NO_DEFAULT), ("three", _NO_DEFAULT)],
+            ":",
+        )
+        namespace = parser.parse_args("1 2 3".split())
+        self.assertEqual(namespace.one, 1)
+        self.assertEqual(namespace.two, 2)
+        self.assertEqual(namespace.three, 3)
+
     def test_get_arg_parser_with_default_value(self):
         parser = _get_arg_parser(
             parse_me_full_docstring,
-            [str, int],
-            [("one", NoDefault), ("two", NoDefault), ("three", 12)],
+            {"one": str, "two": int, "three": int},
+            [("one", _NO_DEFAULT), ("two", _NO_DEFAULT), ("three", 12)],
             ":",
         )
         namespace = parser.parse_args("yes 42".split())
@@ -305,8 +315,8 @@ class TestCore(unittest.TestCase):
     def test_get_arg_parser_without_default_value(self):
         parser = _get_arg_parser(
             parse_me_full_docstring,
-            [str, int],
-            [("one", NoDefault), ("two", NoDefault), ("three", 12)],
+            {"one": str, "two": int, "three": int},
+            [("one", _NO_DEFAULT), ("two", _NO_DEFAULT), ("three", 12)],
             ":",
         )
         namespace = parser.parse_args("no 12 --three=23".split())
@@ -317,8 +327,8 @@ class TestCore(unittest.TestCase):
     def test_get_arg_parser_required_arguments(self):
         parser = _get_arg_parser(
             parse_me_full_docstring,
-            [str, int],
-            [("one", NoDefault), ("two", NoDefault), ("three", 12)],
+            {"one": str, "two": int, "three": int},
+            [("one", _NO_DEFAULT), ("two", _NO_DEFAULT), ("three", 12)],
             ":",
         )
         with captured_output():
@@ -329,8 +339,8 @@ class TestCore(unittest.TestCase):
     def test_get_arg_parser_argument_type(self):
         parser = _get_arg_parser(
             parse_me_full_docstring,
-            [str, int],
-            [("one", NoDefault), ("two", NoDefault), ("three", 12)],
+            {"one": str, "two": int, "three": int},
+            [("one", _NO_DEFAULT), ("two", _NO_DEFAULT), ("three", 12)],
             ":",
         )
         with captured_output():
@@ -357,52 +367,66 @@ class TestCore(unittest.TestCase):
 
     def test_check_types_not_enough_types_provided(self):
         self.assertRaises(
-            ParseThisError, _check_types, "function", [], ["i_dont_have_a_type"], ()
+            ParseThisException, _check_types, "function", {}, ["i_dont_have_a_type"], ()
         )
 
     def test_check_types_too_many_types_provided(self):
         self.assertRaises(
-            ParseThisError, _check_types, "function", [int, str], ["i_am_alone"], ()
+            ParseThisException,
+            _check_types,
+            "function",
+            {"a": int, "b": int},
+            ["i_am_alone"],
+            (),
         )
 
     def test_check_types_with_default(self):
-        types = [int, str]
         func_args = ["i_am_alone", "i_have_a_default_value"]
         self.assertEqual(
-            _check_types("function", types, func_args, ("default_value",)),
-            (types, func_args),
+            _check_types(
+                "function",
+                {"i_am_an_int": int, "i_have_a_default_value": str},
+                func_args,
+                ("default_value",),
+            ),
+            func_args,
         )
 
     def test_check_types_with_default_type_not_specified(self):
-        types = [int]
         func_args = ["i_am_an_int", "i_have_a_default_value"]
         self.assertEqual(
-            _check_types("function", types, func_args, ("default_value",)),
-            (types, func_args),
+            _check_types(
+                "function", {"i_am_an_int": int}, func_args, ("default_value",)
+            ),
+            func_args,
         )
 
     def test_check_types_remove_self(self):
-        types = [int]
         func_args = ["i_am_an_int", "i_have_a_default_value"]
         self.assertEqual(
             _check_types(
-                "function", [Self] + types, ["self"] + func_args, ("default_value",)
+                "function",
+                {"i_am_an_int": int},
+                ["self"] + func_args,
+                ("default_value",),
             ),
-            ([int], func_args),
+            func_args,
         )
 
     def test_check_types_remove_class(self):
-        types = [int]
         func_args = ["i_am_an_int", "i_have_a_default_value"]
         self.assertEqual(
             _check_types(
-                "function", [Class] + types, ["cls"] + func_args, ("default_value",)
+                "function",
+                {"i_am_an_int": int},
+                ["cls"] + func_args,
+                ("default_value",),
             ),
-            ([int], func_args),
+            func_args,
         )
 
     def test_check_types_no_args(self):
-        self.assertEqual(_check_types("function", [], [], ()), ([], []))
+        self.assertEqual(_check_types("function", {}, [], ()), [])
 
     def test_get_parser_call_method_returns_callable(self):
         call_method = _get_parser_call_method(concatenate_string)
@@ -410,7 +434,7 @@ class TestCore(unittest.TestCase):
 
     def test_get_parser_call_method_raise_on_init(self):
         call_method = _get_parser_call_method(Parseable.__init__)
-        self.assertRaises(ParseThisError, call_method, None)
+        self.assertRaises(ParseThisException, call_method, None)
 
     def test_get_parser_call_method_execution(self):
         call_method = _get_parser_call_method(Parseable.parseable)
